InstanceOutliner

Struct InstanceOutliner 

pub struct InstanceOutliner<F: Field> {
    pub pred_label: Label,
    pub func: Rc<dyn Fn(&mut ConstraintSystem<F>, &[Variable]) -> Result<(), SynthesisError>>,
}
An instance outliner is a strategy for reducing the number of constraints that public input/instance variables are involved in. It does this as follows:

  1. Create new public input variables that correspond to the original input variables
  2. Replace the original input variables with corresponding new witness variables
  3. Enforce equality between the new input variables and the new witness variables
  4. In every constraint that involves the original input variables, replace them with the new witness variables.

Fields

pred_label: Label

The label for the predicate that is used to enforce equality between the new input/instance variables and the new witness variables.

func: Rc<dyn Fn(&mut ConstraintSystem<F>, &[Variable]) -> Result<(), SynthesisError>>

The strategy for outlining the instance variables It takes as input the constraint system, and a map from the new instance variables to the new witness variables.