PolynomialPredicate

Struct PolynomialPredicate 

pub struct PolynomialPredicate<F: Field> {
    pub polynomial: SparsePolynomial<F, SparseTerm>,
}
A polynomial predicat is just a polynomial

Fields

polynomial: SparsePolynomial<F, SparseTerm>

The sparse polynomial for the predicate

Implementations

Source

impl<F: Field> PolynomialPredicate<F>

pub fn new( arity: usize, terms: impl IntoIterator<Item = (F, Vec<(usize, usize)>)>, ) -> Self

Create a new polynomial predicate with a given number of variables (= arity) and number of terms.

Here terms is a list of tuples of the form (variable, power) where variable is the index of the variable and power is the exponent of the variable.

So, for example, if terms is [F::ONE, [(0, 1), (1, 2)]], then the polynomial is x_0 + x_1^2.

Source

impl<F: Field> PolynomialPredicate<F>

This is the implementation of the Predicate trait for PolynomialPredicate. The evaluation of a polynomial predicate is the evaluation of the underlying polynomial and the arity is the number of variables in the polynomial.

pub fn is_satisfied(&self, variables: &[F]) -> bool

Check if the predicate is satisfied by the given variables.

pub fn eval(&self, variables: &[F]) -> F

Evaluate the predicate on the given variables.

pub fn arity(&self) -> usize

Get the arity of the polynomial predicate. For example, the arity of P(x1, x2, …, xn) is n.

pub fn degree(&self) -> usize

Get the degree of the polynomial predicate. For example, the degree of x1 + x2^4 + x3^2 is 4.