Sr1csAdapter

Struct Sr1csAdapter 

pub struct Sr1csAdapter<F: Field> { /* private fields */ }
Helper struct to enable easy conversion from R1CS to SR1CS

Implementations

Source

impl<F: Field> Sr1csAdapter<F>

pub fn evaluate_constraint<'a, LHS, RHS, R>( terms: &'a [(LHS, usize)], assignment: &'a [RHS], ) -> R
where LHS: One + Send + Sync + PartialEq, RHS: Send + Sync + Mul<&'a LHS, Output = RHS> + Copy, R: Zero + Send + Sync + AddAssign<RHS> + Sum,

Computes the inner product of terms with assignment.

pub fn r1cs_to_sr1cs( cs: &ConstraintSystemRef<F>, ) -> Result<ConstraintSystemRef<F>, SynthesisError>

Converts an R1CS constraint system to an SR1CS constraint system.

Panics

Panics if the constraint system does not have the R1CS predicate registered or if there is more than one predicate registered.

pub fn r1cs_to_sr1cs_with_assignment( cs: &mut ConstraintSystem<F>, ) -> Result<ConstraintSystemRef<F>, SynthesisError>

Converts an R1CS constraint system to an SR1CS constraint system, while also converting the R1CS assignment to an equivalent SR1CS assignment.

Panics

Panics is the constraint system does not have the R1CS predicate registered.