Module utils

Module utils 

Functions and data structures needed for working with GR1CS

Modules

error
Different errors and error messages used in the library.
linear_combination
Tools to work with linear combinations.
matrix
Tools to work with matrices.
variable
Tools to work with variables.

Type Aliases

HashBuilder
A type alias for a hash builder that uses the foldhash algorithm.
IndexMap
A type alias for an index map that uses the foldhash algorithm.
Result
A result type specialized to SynthesisError.