pub type HashBuilder = SeedableRandomState;Expand description
A type alias for a hash builder that uses the foldhash algorithm.
Aliased Type
pub struct HashBuilder { /* private fields */ }pub type HashBuilder = SeedableRandomState;A type alias for a hash builder that uses the foldhash algorithm.
pub struct HashBuilder { /* private fields */ }