pub type IndexMap<K, V> = IndexMap<K, V, HashBuilder>;Expand description
A type alias for an index map that uses the foldhash algorithm.
Aliased Type
pub struct IndexMap<K, V> { /* private fields */ }pub type IndexMap<K, V> = IndexMap<K, V, HashBuilder>;A type alias for an index map that uses the foldhash algorithm.
pub struct IndexMap<K, V> { /* private fields */ }