ConstraintSystemRef

Enum ConstraintSystemRef 

pub enum ConstraintSystemRef<F: Field> {
    None,
    CS(Rc<RefCell<ConstraintSystem<F>>>),
}
A shared reference to a constraint system that can be stored in high level variables.

Variants



None

Represents the case where we don’t need to allocate variables or enforce constraints. Encountered when operating over constant values.



CS(Rc<RefCell<ConstraintSystem<F>>>)

Represents the case where we do allocate variables or enforce constraints.

Implementations

Source

impl<F: Field> ConstraintSystemRef<F>

pub fn new(inner: ConstraintSystem<F>) -> Self

Construct a ConstraintSystemRef from a ConstraintSystem.

pub fn get_all_predicates_num_constraints(&self) -> IndexMap<Label, usize>

Returns the number of constraints in each predicate

pub fn get_predicates_num_constraints( &self, predicate_label: &str, ) -> Option<usize>

Returns the number of constraints in the predicate with the given label

pub fn get_all_predicate_arities(&self) -> IndexMap<Label, usize>

Returns the arity of each predicate

pub fn get_predicate_arity(&self, predicate_label: &str) -> Option<usize>

Returns the predicate type of the predicate with the given label

pub fn get_all_predicate_types(&self) -> BTreeMap<Label, Predicate<F>>

Returns the predicate types of each predicate

pub fn get_predicate_type(&self, predicate_label: &str) -> Option<Predicate<F>>

Returns the predicate type of the predicate with the given label

pub fn num_constraints(&self) -> usize

Returns the number of constraints which is the sum of the number of constraints in each predicate. #[inline]

pub fn num_instance_variables(&self) -> usize

Returns the number of instance variables.

pub fn instance_assignment(&self) -> Result<Vec<F>>

Returns the number of instance variables.

pub fn witness_assignment(&self) -> Result<Vec<F>>

Returns the number of instance variables.

pub fn num_variables(&self) -> usize

Returns the number of instance variables.

pub fn num_predicates(&self) -> usize

Returns the number of predicates.

pub fn num_witness_variables(&self) -> usize

Returns the number of witness variables.

pub fn enforce_constraint( &self, predicate_label: &str, lc_vec: impl IntoIterator<Item = Box<dyn FnOnce() -> LinearCombination<F>>, IntoIter: ExactSizeIterator>, ) -> Result<()>

Enforce a constraint in the constraint system. It takes a predicate name and enforces a vector of linear combinations of the length of the arity of the predicate enforces the constraint.

pub fn enforce_constraint_arity_2( &self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint with arity 2.

pub fn enforce_constraint_arity_3( &self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint with arity 3.

pub fn enforce_constraint_arity_4( &self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, d: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint with arity 4.

pub fn enforce_constraint_arity_5( &self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, d: impl FnOnce() -> LinearCombination<F>, e: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint with arity 5.

pub fn enforce_r1cs_constraint( &self, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce an R1CS constraint in the constraint system. On input a, b, and c, this method enforces that a * b = c. If the R1CS predicate has not been already added, this method will add it.

This method is a special case of enforce_constraint and is used to provide a low-effort way to port prior code that assumed that R1CS is the only kind of predicate.

pub fn enforce_sr1cs_constraint( &self, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a SquareR1CS constraint in the constraint system. On input a and b, this method enforces that a^2 = b. If the SquareR1CS predicate has not been added to the constraint system, this method will add it. This method is a special case of enforce_constraint.

pub fn new_lc( &self, lc: impl FnOnce() -> LinearCombination<F>, ) -> Result<Variable>

Obtain a new variable representing the linear combination lc.

pub fn set_mode(&self, mode: SynthesisMode)

Set self.mode to mode. Sets the mode if there exists an underlying ConstraintSystem.

pub fn is_in_setup_mode(&self) -> bool

Check whether self.mode == SynthesisMode::Setup. Returns true if

  1. There is an underlying ConstraintSystem and,
  2. It is in setup mode.
pub fn optimization_goal(&self) -> OptimizationGoal

Check whether this constraint system aims to optimize weight, number of constraints, or neither.

pub fn set_optimization_goal(&self, goal: OptimizationGoal)

Specify whether this constraint system should aim to optimize weight, number of constraints, or neither.

pub fn should_outline_instances(&self) -> bool

Should we outline instances according to the optimization specified in Section 3, Page 11 of Polymath By default, this flag is false.

pub fn set_instance_outliner(&self, outliner: InstanceOutliner<F>)

Specify the strategy for how the instance should be outlined. This should be compatible with the predicates in the constraint system.

pub fn should_construct_matrices(&self) -> bool

Check whether or not self will construct matrices.

pub fn new_input_variable<Func>(&self, f: Func) -> Result<Variable>
where Func: FnOnce() -> Result<F>,

Obtain a variable representing a new public instance input.

pub fn new_witness_variable<Func>(&self, f: Func) -> Result<Variable>
where Func: FnOnce() -> Result<F>,

Obtain a new variable representing a new private witness variable.

pub fn register_predicate( &self, label: &str, predicate: PredicateConstraintSystem<F>, ) -> Result<()>

Register a predicate in the constraint system with a given label.

pub fn remove_predicate(&self, label: &str)

Remove a predicate with the given label from the constraint system.

pub fn has_predicate(&self, predicate_label: &str) -> bool

Checks if there is a predicate with the given label in the constraint

pub fn assigned_value(&self, v: Variable) -> Option<F>

Obtain the assignment corresponding to the Variable v.

pub fn is_satisfied(&self) -> Result<bool>

If self is satisfied, outputs Ok(true). If self is unsatisfied, outputs Ok(false). If self.is_in_setup_mode() or if self == None, outputs Err(()).

pub fn which_is_unsatisfied(&self) -> Result<Option<String>>

If self is satisfied, outputs Ok(None). If self is unsatisfied, outputs Some(s,i), where s is the label of the unsatisfied prediacate and i is the index of the first unsatisfied constraint in that predicate. If self.is_in_setup_mode() or self == None, outputs Err(()).

pub fn finalize(&self)

Finalize the constraint system (either by outlining or inlining, if an optimization goal is set).

pub fn inline_all_lcs(&self)

Naively inlines symbolic linear combinations into the linear combinations that use them.

Useful for standard pairing-based SNARKs where addition gates are cheap. For example, in the SNARKs such as [Groth16] and [Groth-Maller17], addition gates do not contribute to the size of the multi-scalar multiplication, which is the dominating cost.

pub fn or(self, other: Self) -> Self

Returns self if !self.is_none(), otherwise returns other.

pub fn is_none(&self) -> bool

Returns true is self == ConstraintSystemRef::None.

pub fn into_inner(self) -> Option<ConstraintSystem<F>>

Consumes self to return the inner ConstraintSystem<F>. Returns None if Self::CS is None or if any other references to Self::CS exist.

pub fn to_matrices(&self) -> Result<BTreeMap<Label, Vec<Matrix<F>>>>

Get the matrices corresponding to the predicates.and the corresponding set of matrices

pub fn get_lc(&self, var: Variable) -> Option<LinearCombination<F>>

Get the linear combination corresponding to the given lc_index. TODO: This function should ideally return a reference to the linear combination and not clone it.

pub fn make_row(&self, lc: LinearCombination<F>) -> Result<Vec<(F, usize)>>

Given a linear combination, create a row in the matrix

pub fn borrow(&self) -> Option<Ref<'_, ConstraintSystem<F>>>

Obtain an immutable reference to the underlying ConstraintSystem.

Panics

This method panics if self is already mutably borrowed.

pub fn borrow_mut(&self) -> Option<RefMut<'_, ConstraintSystem<F>>>

Obtain a mutable reference to the underlying ConstraintSystem.

Panics

This method panics if self is already mutably borrowed.

pub fn constraint_names(&self) -> Option<Vec<String>>

Get trace information about all constraints in the system