#[non_exhaustive]pub enum Predicate<F: Field> {
Polynomial(PolynomialPredicate<F>),
}Expand description
GR1CS can potentially support different types of predicates For now, we only support polynomial predicates In the future, we can add other types of predicates, e.g. lookup table
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Polynomial(PolynomialPredicate<F>)
A polynomial local predicate. This is the most common predicate that captures high-degree custom gates