pub struct PolynomialPredicate<F: Field> {
pub polynomial: SparsePolynomial<F, SparseTerm>,
}Expand description
A polynomial predicat is just a polynomial
Fields
polynomial: SparsePolynomial<F, SparseTerm>The sparse polynomial for the predicate
Implementations
Sourceimpl<F: Field> PolynomialPredicate<F>
impl<F: Field> PolynomialPredicate<F>
Sourcepub fn new(
arity: usize,
terms: impl IntoIterator<Item = (F, Vec<(usize, usize)>)>,
) -> Self
pub fn new( arity: usize, terms: impl IntoIterator<Item = (F, Vec<(usize, usize)>)>, ) -> Self
Create a new polynomial predicate with a given number of variables (= arity) and number of terms.
Here terms is a list of tuples of the form (variable, power) where
variable is the index of the variable and power is the exponent of the variable.
So, for example, if terms is [F::ONE, [(0, 1), (1, 2)]], then the polynomial is
x_0 + x_1^2.
Sourceimpl<F: Field> PolynomialPredicate<F>
This is the implementation of the Predicate trait for PolynomialPredicate.
The evaluation of a polynomial predicate is the evaluation of the underlying
polynomial and the arity is the number of variables in the polynomial.
impl<F: Field> PolynomialPredicate<F>
This is the implementation of the Predicate trait for PolynomialPredicate. The evaluation of a polynomial predicate is the evaluation of the underlying polynomial and the arity is the number of variables in the polynomial.