ConstraintSystem

Struct ConstraintSystem 

pub struct ConstraintSystem<F: Field> {
    pub assignments: Assignments<F>,
    pub cache_map: Rc<RefCell<BTreeMap<TypeId, Box<dyn Any>>>>,
    /* private fields */
}
A GR1CS ConstraintSystem. Enforces constraints of the form

L_i(⟨M_{i,1}, z⟩ , ⟨M_{i,2}, z⟩ , ..., ⟨M_{i,t_i}, z⟩) = 0

More Information: https://eprint.iacr.org/2024/1245

Fields

assignments: Assignments<F>

Assignments to the input, witness, and lc variables. This is empty if self.mode == SynthesisMode::Setup.

cache_map: Rc<RefCell<BTreeMap<TypeId, Box<dyn Any>>>>

Map for gadgets to cache computation results.

Implementations

Source

impl<F: Field> ConstraintSystem<F>

pub fn new() -> Self

Create a new and empty ConstraintSystem<F>. Note that by default, the constraint system is registered with the R1CS predicate.

pub fn new_ref() -> ConstraintSystemRef<F>

Create a new ConstraintSystemRef<F>.

pub fn get_all_predicates_num_constraints(&self) -> IndexMap<Label, usize>

Returns a mapping from predicate label to number of constraints for that predicate

pub fn get_predicate_num_constraints( &self, predicate_label: &str, ) -> Option<usize>

Returns the number of constraints for a given predicate

pub fn get_all_predicate_arities(&self) -> IndexMap<Label, usize>

Returns a mapping from predicate label to arity for that predicate

pub fn get_predicate_arity(&self, predicate_label: &str) -> Option<usize>

Returns the type of the predicate with the given label

pub fn get_all_predicate_types(&self) -> BTreeMap<Label, Predicate<F>>

Returns a mapping from predicate labels to their types

pub fn get_predicate_type(&self, predicate_label: &str) -> Option<Predicate<F>>

Returns the type of the predicate with the given label

pub fn instance_assignment(&self) -> Result<&[F]>

Returns the assignment to the public input variables of the constraint

pub fn witness_assignment(&self) -> Result<&[F]>

Returns the assignment to the private input variables of the constraint

pub fn num_constraints(&self) -> usize

Returns the number of constraints which is the sum of the number of constraints in each predicate.

pub fn num_instance_variables(&self) -> usize

Returns the number of instance variables.

pub fn num_witness_variables(&self) -> usize

Returns the number of witness variables.

pub fn num_variables(&self) -> usize

Returns the number of witness variables.

pub fn num_predicates(&self) -> usize

Returns the number of predicates in the constraint system

pub fn enforce_constraint( &mut self, predicate_label: &str, lcs: impl IntoIterator<Item = Box<dyn FnOnce() -> LinearCombination<F>>, IntoIter: ExactSizeIterator>, ) -> Result<()>

Enforce a constraint in the constraint system. It takes a predicate name and enforces a vector of linear combinations of the length of the arity of the predicate enforces the constraint.

pub fn enforce_constraint_arity_2( &mut self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint for a predicate with arity 2.

pub fn enforce_constraint_arity_3( &mut self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint for a predicate with arity 3.

pub fn enforce_constraint_arity_4( &mut self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, d: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint for a predicate with arity 4.

pub fn enforce_constraint_arity_5( &mut self, predicate_label: &str, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, d: impl FnOnce() -> LinearCombination<F>, e: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint for a predicate with arity 5.

pub fn enforce_r1cs_constraint( &mut self, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, c: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint in the constraint system. It takes a predicate name and enforces a vector of linear combinations of the length of the arity of the predicate enforces the constraint.

pub fn enforce_sr1cs_constraint( &mut self, a: impl FnOnce() -> LinearCombination<F>, b: impl FnOnce() -> LinearCombination<F>, ) -> Result<()>

Enforce a constraint in the constraint system. It takes a predicate name and enforces a vector of linear combinations of the length of the arity of the predicate enforces the constraint.

pub fn new_lc( &mut self, lc: impl FnOnce() -> LinearCombination<F>, ) -> Result<Variable>

Adds a new linear combination to the constraint system.

pub fn set_mode(&mut self, mode: SynthesisMode)

Set self.mode to mode.

pub fn is_in_setup_mode(&self) -> bool

Check whether self.mode == SynthesisMode::Setup. Returns true if

  1. There is an underlying ConstraintSystem, and
  2. It is in setup mode.
pub fn optimization_goal(&self) -> OptimizationGoal

Check whether this constraint system aims to optimize weight, number of constraints, or neither.

pub fn set_optimization_goal(&mut self, goal: OptimizationGoal)

Specify whether this constraint system should aim to optimize weight, number of constraints, or neither.

pub fn should_construct_matrices(&self) -> bool

Check whether or not self will construct matrices.

pub fn should_generate_lc_assignments(&self) -> bool

Check whether or not self will construct matrices.

pub fn new_input_variable<Func>(&mut self, f: Func) -> Result<Variable>
where Func: FnOnce() -> Result<F>,

Obtain a variable representing a new public instance input.

pub fn new_witness_variable<Func>(&mut self, f: Func) -> Result<Variable>
where Func: FnOnce() -> Result<F>,

Obtain a variable representing a new private witness input.

pub fn register_predicate( &mut self, predicate_label: &str, predicate: PredicateConstraintSystem<F>, ) -> Result<()>

Register a predicate in the constraint system with a given label.

pub fn remove_predicate(&mut self, predicate_label: &str)

Remove the predicate with the given label from the constraint system.

pub fn has_predicate(&self, predicate_label: &str) -> bool

check if there is a predicate with the given label

pub fn assigned_value(&self, v: Variable) -> Option<F>

Obtain the assignment corresponding to the Variable v.

pub fn is_satisfied(&self) -> Result<bool>

If self is satisfied, outputs Ok(true). If self is unsatisfied, outputs Ok(false). If self.is_in_setup_mode() or if self == None, outputs Err(()).

pub fn which_is_unsatisfied(&self) -> Result<Option<String>>

If self is satisfied, outputs Ok(None). If self is unsatisfied, outputs Some(s,i), where s is the label of the unsatisfied prediacate and i is the index of the first unsatisfied constraint in that predicate. If self.is_in_setup_mode() or self == None, outputs Err(()).

pub fn finalize(&mut self)

Finalize the constraint system (either by outlining or inlining, if an optimization goal is set).

pub fn inline_all_lcs(&mut self)

Naively inlines symbolic linear combinations into the linear combinations that use them.

Useful for standard pairing-based SNARKs where addition gates are cheap. For example, in the SNARKs such as [Groth16] and [Groth-Maller17], addition gates do not contribute to the size of the multi-scalar multiplication, which is the dominating cost.

pub fn to_matrices(&self) -> Result<BTreeMap<Label, Vec<Matrix<F>>>>

Get the matrices corresponding to the predicates.and the corresponding set of matrices

pub fn get_lc(&self, var: Variable) -> LinearCombination<F>

Get the linear combination corresponding to the given lc_index.

pub fn perform_instance_outlining( &mut self, outliner: InstanceOutliner<F>, ) -> Result<()>

Outlines the instances in the constraint system.

This function creates a new witness variable for each instance variable and uses these witness variables in the constraints instead of instance variables. This technique is useful for verifier succinctness in some SNARKs like Garuda, Pari and PolyMath. After the function call, instances are only used in the C matrix of r1cs