Enum ark_relations::r1cs::SynthesisError[][src]

pub enum SynthesisError {
    MissingCS,
    AssignmentMissing,
    DivisionByZero,
    Unsatisfiable,
    PolynomialDegreeTooLarge,
    UnexpectedIdentity,
    MalformedVerifyingKey,
    UnconstrainedVariable,
}

This is an error that could occur during circuit synthesis contexts, such as CRS generation, proving or verification.

Variants

MissingCS

During synthesis, we tried to allocate a variable when ConstraintSystemRef was None.

AssignmentMissing

During synthesis, we lacked knowledge of a variable assignment.

DivisionByZero

During synthesis, we divided by zero.

Unsatisfiable

During synthesis, we constructed an unsatisfiable constraint system.

PolynomialDegreeTooLarge

During synthesis, our polynomials ended up being too high of degree

UnexpectedIdentity

During proof generation, we encountered an identity in the CRS

MalformedVerifyingKey

During verification, our verifying key was malformed.

UnconstrainedVariable

During CRS generation, we observed an unconstrained auxiliary variable