Enum ark_relations::r1cs::SynthesisError [−][src]
This is an error that could occur during circuit synthesis contexts, such as CRS generation, proving or verification.
Variants
During synthesis, we tried to allocate a variable when
ConstraintSystemRef was None.
During synthesis, we lacked knowledge of a variable assignment.
During synthesis, we divided by zero.
During synthesis, we constructed an unsatisfiable constraint system.
During synthesis, our polynomials ended up being too high of degree
During proof generation, we encountered an identity in the CRS
During verification, our verifying key was malformed.
During CRS generation, we observed an unconstrained auxiliary variable