Enum ark_relations::r1cs::SynthesisMode[][src]

pub enum SynthesisMode {
    Setup,
    Prove {
        construct_matrices: bool,
    },
}

Defines the mode of operation of a ConstraintSystem.

Variants

Setup

Indicate to the ConstraintSystem that it should only generate constraint matrices and not populate the variable assignments.

Prove

Indicate to the ConstraintSystem that it populate the variable assignments. If additionally construct_matrices == true, then generate the matrices as in the Setup case.

Fields of Prove

construct_matrices: bool

If construct_matrices == true, then generate the matrices as in the Setup case.