Struct ark_relations::r1cs::ConstraintMatrices [−][src]
pub struct ConstraintMatrices<F: Field> { pub num_instance_variables: usize, pub num_witness_variables: usize, pub num_constraints: usize, pub a_num_non_zero: usize, pub b_num_non_zero: usize, pub c_num_non_zero: usize, pub a: Matrix<F>, pub b: Matrix<F>, pub c: Matrix<F>, }
The A, B and C matrices of a Rank-One ConstraintSystem.
Also contains metadata on the structure of the constraint system
and the matrices.
Fields
num_instance_variables: usizeThe number of variables that are “public instances” to the constraint system.
num_witness_variables: usizeThe number of variables that are “private witnesses” to the constraint system.
num_constraints: usizeThe number of constraints in the constraint system.
a_num_non_zero: usizeThe number of non_zero entries in the A matrix.
b_num_non_zero: usizeThe number of non_zero entries in the B matrix.
c_num_non_zero: usizeThe number of non_zero entries in the C matrix.
a: Matrix<F>The A constraint matrix. This is empty when
self.mode == SynthesisMode::Prove { construct_matrices = false }.
b: Matrix<F>The B constraint matrix. This is empty when
self.mode == SynthesisMode::Prove { construct_matrices = false }.
c: Matrix<F>The C constraint matrix. This is empty when
self.mode == SynthesisMode::Prove { construct_matrices = false }.