pub struct Sr1csAdapter<F: Field> { /* private fields */ }Expand description
Helper struct to enable easy conversion from R1CS to SR1CS
Implementations
Sourceimpl<F: Field> Sr1csAdapter<F>
impl<F: Field> Sr1csAdapter<F>
Sourcepub fn evaluate_constraint<'a, LHS, RHS, R>(
terms: &'a [(LHS, usize)],
assignment: &'a [RHS],
) -> R
pub fn evaluate_constraint<'a, LHS, RHS, R>( terms: &'a [(LHS, usize)], assignment: &'a [RHS], ) -> R
Computes the inner product of terms with assignment.
Sourcepub fn r1cs_to_sr1cs(
cs: &ConstraintSystemRef<F>,
) -> Result<ConstraintSystemRef<F>, SynthesisError>
pub fn r1cs_to_sr1cs( cs: &ConstraintSystemRef<F>, ) -> Result<ConstraintSystemRef<F>, SynthesisError>
Converts an R1CS constraint system to an SR1CS constraint system.
Panics
Panics if the constraint system does not have the R1CS predicate registered or if there is more than one predicate registered.
Sourcepub fn r1cs_to_sr1cs_with_assignment(
cs: &mut ConstraintSystem<F>,
) -> Result<ConstraintSystemRef<F>, SynthesisError>
pub fn r1cs_to_sr1cs_with_assignment( cs: &mut ConstraintSystem<F>, ) -> Result<ConstraintSystemRef<F>, SynthesisError>
Converts an R1CS constraint system to an SR1CS constraint system, while also converting the R1CS assignment to an equivalent SR1CS assignment.
Panics
Panics is the constraint system does not have the R1CS predicate registered.