Functions and data structures needed for working with GR1CS
Modules
- error
- Different errors and error messages used in the library.
- linear_
combination - Tools to work with linear combinations.
- matrix
- Tools to work with matrices.
- variable
- Tools to work with variables.
Type Aliases
- Hash
Builder - A type alias for a hash builder that uses the foldhash algorithm.
- Index
Map - A type alias for an index map that uses the foldhash algorithm.
- Result
- A result type specialized to
SynthesisError.